/*= OX822 Metrix ===================================================== */
/* LabWindows Instrument Driver for OX822.                             */
/* Original Release: 20/01/99		                                   */
/* Originally written in C                                             */
/* Modification History:                                               */
/*=====================================================================*/
#include "formatio.h"
#include "rs232.h"
#include "ox822.h"

/*= STATIC VARIABLES ======================================================*/
/* port contains the number of the port opened for the instrument module*/
/* vert_mode contains the current vertical mode of the instrument.         */
/* ox822_err: the error variable for the instrument module                 */
/*=========================================================================*/
extern int port;
int	vert_mode;
int ox822_err;


/*= UTILITY ROUTINES ======================================================*/
int ox822_invalid_short_range (short val, short min, short max, int err_code);
int ox822_invalid_integer_range (int val, int min, int max, int err_code);
int ox822_invalid_longint_range (long val, long min, long max, int err_code);
int ox822_invalid_real_range (double val, double min, double max, int err_code);
int ox822_device_closed (void);
int ox822_send_command (int, int);

/*=========================================================================*/
/* This function opens a com port for the instrument module, queries for   */
/* ID, and initializes the instrument to a known state.                    */
/*=========================================================================*/
int ox822_init (int rs_port)
{
    char identifier[3];

    ox822_err = 0;
    if (ox822_invalid_integer_range (rs_port,1,8,-1) != 0){
        return ox822_err;
    }

    /*   Open And Configure RS232  */
    OpenComConfig (rs_port, "", 9600, 0, 8, 1, 0, 0);
    if (rs232err != 0) {
        ox822_err = rs232err+300;
        return ox822_err;
    }
    /*   Time-out : 1 Sec   */
    SetComTime (rs_port, 1.0);
    if (rs232err != 0) {
        ox822_err = rs232err+300;
        return ox822_err;
    }
    /*   Sets XON/XOFF ON   */
    SetXMode (rs_port, 1);
    if (rs232err != 0) {
        ox822_err = rs232err+300;
        return ox822_err;
    }
    port = rs_port;

    /*   Identify instrument   */
    if (ox822_send_command (OX822_ASK_ECHO, 0) !=0 ){
        ox822_close();
        return ox822_err;
    }
    ComRdTerm (port, identifier, 03, 04);
    if (identifier[1]!=0x37){
        ox822_close();
        return 223;
    }

    /*   Sets the Instrument with default values  */
    /*    Beam Finder   :   OFF             */
    /*    Vertical mode :   ALT             */
    /*    CH1           : AC    1V/Div      */
    /*    CH2           : AC    1V/Div      */
    /*    Time base     : x10 OFF,0.1MS/Div,DELAY OFF                       */
    /*    Trigger       : PTP ON,SLOPE +,AUTO ON,SOURCE CH1,COUPLING DC     */

    vert_mode = 1; /* ALT */
    if (ox822_display (1) != 0){
        return ox822_err;
    }
    if (ox822_channel (0, 0, 7,1) != 0){
        return ox822_err;
    }
    if (ox822_channel (1, 0, 7,1) !=0){
        return ox822_err;
    }
    if (ox822_timebase (0, 10, 0) != 0){
        return ox822_err;
    }
    if (ox822_trigger (1, 0, 1, 0, 0) != 0){
        return ox822_err;
    }
    return ox822_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument Setup subroutine                         */
/*---------------------------------------------------------------------*/
int ox822_setup (int rw_config, char *file)
{
    int     file_id;
    short   i,tab_fonct[20];
    char    buff[20];

	tab_fonct[0] = OX822_INIT;
	tab_fonct[1] = OX822_RANGE_CH1;
	tab_fonct[2] = OX822_RANGE_CH2;
	tab_fonct[3] = OX822_RANGE_TIME_BASE;
	tab_fonct[4] = OX822_TRIGGER_SLOPE;
	tab_fonct[5] = OX822_COUPLING_CH1;
	tab_fonct[6] = OX822_DIFF_CH1;
	tab_fonct[7] = OX822_COUPLING_CH2;
	tab_fonct[8] = OX822_DIFF_CH2;
	tab_fonct[9] = OX822_EXP10;
	tab_fonct[10] = OX822_TRIGGER_AUTO;
	tab_fonct[11] = OX822_TRIGGER_PTP;
	tab_fonct[12] = OX822_TRIGGER_COUPLING;
	tab_fonct[13] = OX822_TRIGGER_SOURCE;
	tab_fonct[14] = OX822_TB_DELAY;
	tab_fonct[15] = OX822_TIME_BASE_DELAY;
	tab_fonct[16] = OX822_VERTICAL_MODE;

    ox822_err = 0;
    if (ox822_device_closed() != 0 ){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (rw_config,0,1,-1) != 0){
        return ox822_err;
    }
    if (!rw_config)  {
        /* Save */
        file_id = OpenFile (file, 2, 0, 1);
        if (file_id ==  -1)  {
            ox822_err =  -2;
            return ox822_err;
        }
        /* Input Buffer Empty */
        FlushInQ (port);
        if (rs232err != 0) {
            ox822_err = rs232err+300;
            return ox822_err;
        }
        if (ox822_send_command (OX822_READ_SETUP, 0) != 0 ){
            return ox822_err;
        }
        i = 0;
        while (i < 17)  {
            ComRdByte (port);
            if (rs232err != 0)  {
                ox822_err = rs232err+300;
                return ox822_err;
            }
            buff[i] = ComRdByte (port);
            if (rs232err != 0)  {
                ox822_err = rs232err+300;
                return ox822_err;
            }
            ComRdByte (port);
            if (rs232err != 0)  {
                ox822_err = rs232err+300;
                return ox822_err;
            }
            i++;
        }
        if (WriteFile (file_id, buff, 17) == -1){
            ox822_err =  230;
            return ox822_err;
        }
        if (CloseFile (file_id) == -1){
            ox822_err = -2 ;
            return ox822_err;
        }
    }
    else  {
        /* Recall */
        file_id = OpenFile (file, 1, 0, 1);
        if (file_id ==  -1)  {
            ox822_err =  -2;
            return ox822_err;
        }
        if (ReadFile (file_id, buff, 17) ==  -1)  {
            ox822_err =  231;
            return ox822_err;
        }

        i = 0;
        while (i < 17)  {
            if (ox822_send_command (tab_fonct[i], buff[i] - 0x30) != 0 ){
                return ox822_err;
            }
            i++;
        }
        if (CloseFile (file_id) == -1 ){
            ox822_err =  -2;
            return ox822_err;
        }

    }
    return ox822_err;
}
/*---------------------------------------------------------------------*/
/*                  Display settings subroutine                        */
/*---------------------------------------------------------------------*/
int ox822_display (int mod)
{
    ox822_err = 0;
    if (ox822_device_closed() != 0 ){
        return ox822_err ;
    }
    if (ox822_invalid_integer_range (mod,0,4,-1) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_VERTICAL_MODE, mod) != 0){
        return ox822_err;
    }
    vert_mode = mod;
    return ox822_err;
}
/*---------------------------------------------------------------------*/
/*                 Channel Settings subroutine                         */
/*---------------------------------------------------------------------*/
int ox822_channel (int channel_num, int coupling, int sensibility,int nor_diff)
{
    ox822_err = 0;
    if (ox822_device_closed() != 0 ){
        return ox822_err ;
    }
    if (ox822_invalid_integer_range (channel_num,0,1,-1) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (coupling,0,2,-2) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (sensibility,0,14,-3) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (nor_diff,0,1,-4) != 0){
        return ox822_err;
    }
    if (!channel_num){
        if (ox822_send_command (OX822_COUPLING_CH1, coupling) != 0)
            return ox822_err;
    }
    else{
        if (ox822_send_command (OX822_COUPLING_CH2, coupling) != 0)
            return ox822_err;
    }
    if (!channel_num){
        if (ox822_send_command (OX822_RANGE_CH1, sensibility) != 0)
            return ox822_err;
    }
    else{
        if (ox822_send_command (OX822_RANGE_CH2, sensibility) != 0){
            return ox822_err;
        }
    }
    /* Input NOR/DIFF */
    if (ox822_send_command (OX822_DIFF_CH1+channel_num, nor_diff) !=0){
        return ox822_err;
    }

    return ox822_err;
}
/*---------------------------------------------------------------------*/
/*                     Time base subroutine                            */
/*---------------------------------------------------------------------*/
int ox822_timebase (int expend,int range,int dlay)
{

    ox822_err = 0;
    if (ox822_device_closed() != 0 ){
        return ox822_err ;
    }
    if (ox822_invalid_integer_range (expend,0,1,-1) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (range,0,18,-2) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (dlay,0,2,-3) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_EXP10, expend) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_RANGE_TIME_BASE, range) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_TB_DELAY, dlay) != 0){
        return ox822_err;
    }
    return ox822_err;
}
/*---------------------------------------------------------------------*/
/*                     Trigger subroutine                              */
/*---------------------------------------------------------------------*/
int ox822_trigger (int ptp,int slope,int mod_auto,int source,int coupling)
{
    ox822_err = 0;
    if (ox822_device_closed() != 0 ){
        return ox822_err ;
    }
    if (ox822_invalid_integer_range (ptp,0,1,-1) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (slope,0,1,-2) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (mod_auto,0,1,-3) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (source,0,4,-4) != 0){
        return ox822_err;
    }
    if (ox822_invalid_integer_range (coupling,0,5,-5) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_TRIGGER_SOURCE, source) != 0){
        return ox822_err;
    }
    if (source != 3){
        if (ox822_send_command (OX822_TRIGGER_COUPLING, coupling) != 0){
            return ox822_err;
        }
    }
    /* if SOURCE LINE, COUPLING OFF */
    else {
        if (ox822_send_command (OX822_TRIGGER_COUPLING, 6) != 0){
            return ox822_err;
        }
    }
    if (ox822_send_command (OX822_TRIGGER_PTP, ptp) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_TRIGGER_AUTO, mod_auto) != 0){
        return ox822_err;
    }
    if (ox822_send_command (OX822_TRIGGER_SLOPE, slope) != 0){
        return ox822_err;
    }
    return ox822_err;
}
/*----------------------------------------------------------------------*/
/* This function closes the port for the instrument module and sets the */
/* port to zero.                                                        */
/*----------------------------------------------------------------------*/
int ox822_close (void)
{
	ox822_err=0;
    /*  Check for device closed */
    if (ox822_device_closed() !=0){
        return ox822_err;
    }

    /* Wait for Empty output buffer */
    while ( GetOutQLen(port) != 0) {};
    /* Close the com port.  If error, set ox822_err = rs232err+300.*/
    CloseCom(port);
    if (rs232err != 0) {
        ox822_err = rs232err+300;
        return ox822_err;
    }
    port = 0;
    return ox822_err;
}
/* = UTILITY ROUTINES =====================================================*/
/*=========================================================================*/
/* Function: Invalid Short Range                                           */
/* Purpose:  This function checks a short to see if it lies between a      */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox822_invalid_short_range (short val, short min, short max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox822_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox822_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox822_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int ox822_invalid_longint_range (long val, long min, long max, int err_code)
{
    if (val < min || val > max) {
        ox822_err = err_code;
        return -1;
    }
    return 0;
}
/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox822_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox822_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a 1 is       */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int ox822_device_closed (void)
{
    if (port== 0){
        ox822_err = 232;
        return -1;
    }
    return 0 ;

}
/*=========================================================================*/
/* Function: Send Command                                                  */
/* Purpose: This function sends a command to the scope. If the scope is not*/
/*          initialized, an error 232 is returned.                         */
/*=========================================================================*/
int ox822_send_command (int function, int parameter)
{
    ox822_err = 0;
    if (port != 0)  {
        parameter += 0x30;
        if (ComWrtByte (port, function) ==0){
            ox822_err = 230;
            return ox822_err ;
        }
        if (ComWrtByte (port, parameter) == 0){
            ox822_err = 230;
            return ox822_err ;
        }
        if (ComWrtByte (port, 04) ==0){
            ox822_err = 230;
            return ox822_err ;
        }
    }
    else{
        ox822_err = 232;
    }
    return ox822_err ;
}
/*= THE END ===============================================================*/
